#!/bin/bash
##
## Pipeline script for checking find the runParameters.xml file for a sequencing
## run. 
## 
## Environment variables that should be defined before calling this script
##  -AllRunArchives: White-space separated list of locations to search for sequencing data
##  -BARCODE: Barcode of the flow cell that we want to find information for
##
## The output is the full name of the folder, a date and the contents of the XML file.

# Try to find a folder inside run-archive that has the barcode in the name
# The folder may not yet exist so a missing folder is not an error
DATA_FOLDER=`find ${AllRunArchives} -maxdepth 2 -iname "*${BARCODE}*" -type d -print 2> /dev/null || true`

# Fail if more than one folder is found
readarray -t lines <<<"${DATA_FOLDER}"
if [ ! "${#lines[@]}" -eq 1 ]; then
	echo "Found ${#lines[@]} data folders for flow cell ${BARCODE} in [${AllRunArchives}]" 1>&2
	echo ${DATA_FOLDER} 1>&2
	exit 1
fi

if [ -z "${DATA_FOLDER}" ]; then
   echo "Can't find data folder for flow cell ${BARCODE} in [${AllRunArchives}]" 1>&2
   exit 1
fi
if [ -f "${DATA_FOLDER}/RunParameters.xml" ]; then
     RUN_PARAMETERS=RunParameters.xml
elif [ -f "${DATA_FOLDER}/runParameters.xml" ]; then
     RUN_PARAMETERS=runParameters.xml
else
   echo "Can't find RunParameters.xml or runParameters.xml in folder ${DATA_FOLDER}" 1>&2
   exit 1
fi
if [ -f "${DATA_FOLDER}/RunCompletionStatus.xml" ]; then
     RUN_COMPLETE=RunCompletionStatus.xml
elif [ -f "${DATA_FOLDER}/RTAComplete.txt" ]; then
     RUN_COMPLETE=RTAComplete.txt
fi
echo "${DATA_FOLDER}"
if [ -f "${DATA_FOLDER}/${RUN_COMPLETE}" ]; then
   date +'%Y%m%d %H%M' -r "${DATA_FOLDER}/${RUN_COMPLETE}"
else
   echo 
fi
cat "${DATA_FOLDER}/${RUN_PARAMETERS}"
