var Remote = function()
{
	var remote = {};
	var debug = 0;
	
	remote.initPage = function()
	{
		remote.getHostInfo(false);
		Buttons.addClickHandler('btnReload', remote.reloadConfiguration);
	}
	
	remote.reloadConfiguration = function()
	{
		remote.getHostInfo(true);
	}
	
	remote.getHostInfo = function(reload)
	{
		var rows = Doc.element('all-rows');
		while (rows.lastChild)
		{
			rows.removeChild(rows.lastChild);
		}
		var template = Doc.element('row-template');
		var cols = template.getElementsByTagName('th');
		
		var wait = document.createElement('tr');
		wait.innerHTML = '<td colspan="'+cols.length+'"><div class="messagecontainer note">Loading remote host information. Please wait...</div></td>';
		rows.appendChild(wait);

		var url = '../../OpenGrid.servlet?ID='+App.getSessionId();
		url += '&cmd=GetHostInfo&includeUnconfigured=1&includeNotConnected=1';
		if (reload) url += '&reload=1';
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, remote.onHostInfoLoaded, remote.onHostInfoLoaded);
		request.send(null);
	}
	
	remote.onHostInfoLoaded = function(request)
	{
		if (debug) App.debug(Strings.encodeTags(request.responseText));
		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		var template = Doc.element('row-template');
		var cols = template.getElementsByTagName('th');
		
		var rows = Doc.element('all-rows');
		while (rows.lastChild)
		{
			rows.removeChild(rows.lastChild);
		}

		if (error)
		{
			var wait = document.createElement('tr');
			wait.innerHTML = '<td colspan="'+cols.length+'"><div class="messagecontainer error">'+Strings.encodeTags(error)+'</div></td>';
			rows.appendChild(wait);
			return;
		}
		
		var hosts = response.hosts;
		for (var hostNo = 0; hostNo < hosts.length; hostNo++)
		{
			var host = hosts[hostNo];
			var connectionInfo = host.connection;
			var hostInfo = host.clusterInfo.hostInfo;
			var openGridInfo = host.clusterInfo.openGridInfo;
			
			var row = document.createElement('tr');
			row.className = hostNo % 2 == 0 ? 'evenrow highlight' : 'oddrow highlight';
			
			for (var colNo = 0; colNo < cols.length; colNo++)
			{
				var cell = document.createElement('td');
				cell.className = 'cell';
				var colId = cols[colNo].id;
				if (colId == 'hosts.name')
				{
					cell.innerHTML = '<span class="link">'+Strings.encodeTags(connectionInfo.name)+'<span>';
					Data.set(cell, 'host-id', host.id)
					Events.addEventHandler(cell, 'click', remote.hostOnClick);
				}
				else if (colId == 'hosts.server')
				{
					cell.innerHTML = Strings.encodeTags(connectionInfo.user + '@' + connectionInfo.address + ':' + connectionInfo.port);
				}
				else if (colId == 'hosts.timeDiff')
				{
					cell.innerHTML = host.clusterInfo.timeAdjustment + ' seconds';
				}
				else if (colId == 'hosts.ogs')
				{
					var html;
					if (openGridInfo.exitStatus == 0)
					{
						html = Strings.encodeTags(openGridInfo.stdout);
					}
					else
					{
						html = '<div class="messagecontainer error">['+openGridInfo.exitStatus+'] '+Strings.encodeTags(openGridInfo.stderr)+'</div>';
					}
					cell.innerHTML = html;
				}
				else if (colId == 'hosts.info')
				{
					var html;
					if (hostInfo.exitStatus == 0)
					{
						html = Strings.encodeTags(hostInfo.stdout);
					}
					else
					{
						html = '<div class="messagecontainer error">['+hostInfo.exitStatus+'] '+Strings.encodeTags(hostInfo.stderr)+'</div>';
					}
					cell.innerHTML = html;
				}
				else if (colId == 'hosts.status')
				{
					var img = 'ok.png';
					if (host.clusterInfo.error) 
					{
						img = 'error.png';
					}
					else if (!host.reggeConfigExists)
					{
						img = 'warning.png';
					}
					cell.className = 'host-status';
					cell.innerHTML = '<img src="../../images/'+img + '">';
				}
				else if (colId == 'hosts.configured')
				{
					cell.innerHTML = host.reggeConfigExists ? 'Yes' : 'No';
				}
				row.appendChild(cell);
			}
			
			rows.appendChild(row);
		}
		
	}
	
	remote.hostOnClick = function(event)
	{
		var hostId = Data.get(event.currentTarget, 'host-id');
		var url = '../../../opengrid.jar/view-host.jsp?ID='+App.getSessionId();
		url += '&hostId='+encodeURIComponent(hostId);
		location.href = url;
	}
	
	return remote;
}();

Doc.onLoad(Remote.initPage);
