var Align = function()
{
	var align = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeMergedSequences = null;
	var annotationTypePipeline = null;
	var clusterIsValid = false;
	var arrayDesignIsValid = false;
	var manuallySelected = [];
	
	// Page initialization
	align.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectMergedSequences', align.selectMergedSequences);
		Events.addEventHandler('mergedSequences', 'base-selected', align.setMergedSequenceCallback);
		Events.addEventHandler('mergedSequences', 'change', align.mergedSequencesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', align.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', align.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', align.validateStep2);
		Events.addEventHandler('clusters', 'change', align.clusterOnChange);
		Events.addEventHandler('maskingSoftware', 'change', align.softwareOnChange);
		Events.addEventHandler('alignSoftware', 'change', align.softwareOnChange);
		Events.addEventHandler('cufflinksSoftware', 'change', align.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', align.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Cufflinks.servlet?ID='+App.getSessionId();
		url += '&cmd=GetMergedSequencesForAlignment';
		Wizard.asyncJsonRequest(url, align.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	align.initializeStep1 = function(response)
	{
		var mergedSequences = response.mergedSequences;
		var frm = document.forms['reggie'];	

		if (mergedSequences != null && mergedSequences.length > 0)
		{
			var firstPlate = null;
			for (var msNo=0; msNo < mergedSequences.length; msNo++)
			{
				var ms = mergedSequences[msNo];
				var libPlate = ms.lib.bioWell ? ms.lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				var option = align.createListOption(msNo+1, ms, ms.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
				frm.mergedSequences.options[frm.mergedSequences.length] = option;
			}
			Events.sendChangeEvent('mergedSequences');
		}
		else
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', 'No new sequences available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.mergedSequences.focus();
	}
	
	align.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	align.selectMergedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.mergedSequences.disabled) return;
		
		if (subtypeMergedSequences == null)
		{
			subtypeMergedSequences = Reggie.getSubtypeInfo('MERGED_SEQUENCES');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeMergedSequences.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'=RNAseq';
		Dialogs.selectItem('DERIVEDBIOASSAY', 'mergedSequences', 1, url);
	}

	align.setMergedSequenceCallback = function(event)
	{
		var ms = event.detail;
		var opt = Reggie.getListOption('mergedSequences', ms.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = ms.id;
		}
		
		if (ms.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../Cufflinks.servlet?ID='+App.getSessionId();
			url += '&cmd=GetMergedSequencesForAlignment';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, align.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('mergedSequences');
		}
	}
	
	align.manuallySelected = function(response)
	{
		var mergedSequences = response.mergedSequences;
		var frm = document.forms['reggie'];	

		if (mergedSequences != null && mergedSequences.length > 0)
		{
			var offset = frm.mergedSequences.length+1;
			for (var msNo=0; msNo < mergedSequences.length; msNo++)
			{
				var ms = mergedSequences[msNo];
				var option = align.createListOption(msNo+offset, ms, ms.DO_NOT_USE == null);
				frm.mergedSequences.options[frm.mergedSequences.length] = option;
			}
			Events.sendChangeEvent('mergedSequences');
		}
	}

	align.createListOption = function(index, mergedItem, selected)
	{
		var lib = mergedItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += mergedItem.name;
		
		if (mergedItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(mergedItem.DO_NOT_USE+': '+mergedItem.DO_NOT_USE_COMMENT);
		}
		else if (mergedItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, mergedItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.mergedSequences = mergedItem;
		return option;
	}
	
	align.mergedSequencesOnChange = function()
	{
		var frm = document.forms['reggie'];
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;
		
		Wizard.setInputStatus('mergedSequences');
		Wizard.hideGoNextConfirmation();
		
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) 
			{
				numSelected++;
				
				var ms = frm.mergedSequences[msNo].mergedSequences;
				if (ms.DO_NOT_USE) numDoNotUse++;
				
				if (ms.pipeline && ms.pipeline != 'RNAseq')
				{
					invalidPipeline = Strings.encodeTags(ms.name+' ('+ms.pipeline)+') is not intended for the RNAseq pipeline.';
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('mergedSequences', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('mergedSequences', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('mergedSequences', 'valid');
		}
	}

	
	// --- Step 2 -----------------------------------
	align.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('MASKING_PROTOCOL', 'maskingProtocol');
		Reggie.loadSoftware('MASKING_SOFTWARE', 'maskingSoftware', 'PARAMETER_SET');
		Reggie.loadProtocols('ALIGNMENT_PROTOCOL', 'alignProtocol', 'ALIGNMENT_TYPE', 'Tophat');
		Reggie.loadSoftware('ALIGNMENT_SOFTWARE', 'alignSoftware', 'ALIGNMENT_TYPE,PARAMETER_SET', 'Tophat');
		Reggie.loadProtocols('FEATURE_EXTRACTION_PROTOCOL', 'cufflinksProtocol', 'EXPRESSION_TYPE', 'Cufflinks');
		Reggie.loadSoftware('FEATURE_EXTRACTION_SOFTWARE', 'cufflinksSoftware', 'EXPRESSION_TYPE,PARAMETER_SET', 'Cufflinks');
		
		// Load array design
		var url = '../Cufflinks.servlet?ID='+App.getSessionId() + '&cmd=GetArrayDesigns';
		Doc.addClass('arrayDesign', 'list-loading');
		frm.arrayDesign[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, align.arrayDesignsLoaded);

		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=legacy';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, align.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	align.validateStep2 = function()
	{
		var valid = clusterIsValid;
		valid &= arrayDesignIsValid;
		if (!valid) event.preventDefault();
	}
	
	align.arrayDesignsLoaded = function(response)
	{
		Doc.removeClass('arrayDesign', 'list-loading');
		var frm = document.forms['reggie'];
		var designs = response.arrayDesigns;
		frm.arrayDesign.length = 0;
		for (var i = 0; i < designs.length; i++)
		{
			var design = designs[i];
			var option = new Option(design.name + ' ('+design.numFeatures+')', design.id, design.isDefault, design.isDefault);
			option.arrayDesign = design;
			frm.arrayDesign[frm.arrayDesign.length] = option;
		}
		if (frm.arrayDesign.length == 0)
		{
			Wizard.setInputStatus('arrayDesign', 'invalid', 'No available array designs');
			arrayDesignIsValid = false;
		}
		else
		{
			Wizard.setInputStatus('arrayDesign', 'valid');
			arrayDesignIsValid = true;
		}
	}

	align.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			align.clusterOnChange();
		}
	}
	
	align.softwareOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, align.parameterSetInfoLoaded);
		}
	}
	
	align.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	align.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	

	align.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.maskingSoftware = parseInt(frm.maskingSoftware.value);
		submitInfo.maskingProtocol = parseInt(frm.maskingProtocol.value);
		submitInfo.alignSoftware = parseInt(frm.alignSoftware.value);
		submitInfo.alignProtocol = parseInt(frm.alignProtocol.value);
		submitInfo.arrayDesign = parseInt(frm.arrayDesign.value);
		submitInfo.cufflinksSoftware = parseInt(frm.cufflinksSoftware.value);
		submitInfo.cufflinksProtocol = parseInt(frm.cufflinksProtocol.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var mergedSequences = [];
		submitInfo.mergedSequences = mergedSequences;
			
		for (var msNo = 0; msNo < frm.mergedSequences.length; msNo++)
		{
			if (frm.mergedSequences[msNo].selected) 
			{
				var ms = {};
				ms.id = frm.mergedSequences[msNo].mergedSequences.id;
				mergedSequences[mergedSequences.length] = ms;
			}
		}

		var url = '../Cufflinks.servlet?ID='+App.getSessionId();
		url += '&cmd=StartCufflinks';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, align.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	align.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return align;
}();

Doc.onLoad(Align.initPage);

