var StringTie = function()
{
	var stringtie = {};
	var debug = 0;
	var selectionIsValid = false;
	var subtypeAlignedSequences = null;
	var annotationTypePipeline = null;
	var fileTypeBAM = null;
	
	var clusterIsValid = false;
	var arrayDesignIsValid = false;
	var manuallySelected = [];
	
	// Page initialization
	stringtie.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectAlignedSequences', stringtie.selectAlignedSequences);
		Events.addEventHandler('alignedSequences', 'base-selected', stringtie.setAlignedSequenceCallback);
		Events.addEventHandler('alignedSequences', 'change', stringtie.alignedSequencesOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', stringtie.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', stringtie.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', stringtie.validateStep2);
		Events.addEventHandler('clusters', 'change', stringtie.clusterOnChange);
		Events.addEventHandler('stringtieSoftware', 'change', stringtie.softwareOnChange);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', stringtie.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../StringTie.servlet?ID='+App.getSessionId();
		url += '&cmd=GetAlignedSequencesForStringTie';
		Wizard.asyncJsonRequest(url, stringtie.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	stringtie.initializeStep1 = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var firstPlate = null;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var as = alignedSequences[asNo];
				var libPlate = as.lib.bioWell ? as.lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				var option = stringtie.createListOption(asNo+1, as, as.DO_NOT_USE == null && libPlate != null && libPlate.name==firstPlate)
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
			Events.sendChangeEvent('alignedSequences');
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'No new sequences available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.alignedSequences.focus();
	}
	
	stringtie.validateStep1 = function(event)
	{
		if (!selectionIsValid) event.preventDefault();
	}
	
	stringtie.selectAlignedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.alignedSequences.disabled) return;
		
		if (subtypeAlignedSequences == null)
		{
			subtypeAlignedSequences = Reggie.getSubtypeInfo('ALIGNED_SEQUENCES');
		}
		if (fileTypeBAM == null)
		{
			fileTypeBAM = Reggie.getDataFileTypeInfo('BAM');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		// Reset list
		manuallySelected = [];
		var url = '&resetTemporary=1';
		// Alignments with Hisat gives us 'AlignedSequences' items with 'alignment.bam'
		url += '&tmpfilter:INT:itemSubtype='+subtypeAlignedSequences.id;
		url += '&tmpfilter:STRING:'+encodeURIComponent('¤'+fileTypeBAM.id+'(file.name)')+'=alignment.bam';
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'='+encodeURIComponent('RNAseq/Hisat/StringTie');

		Dialogs.selectItem('DERIVEDBIOASSAY', 'alignedSequences', 1, url);
	}

	stringtie.setAlignedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var opt = Reggie.getListOption('alignedSequences', ms.id);
		if (opt)
		{
			opt.selected = true;
		}
		else
		{
			manuallySelected[manuallySelected.length] = ms.id;
		}
		
		if (ms.remaining == 0 && manuallySelected.length > 0)
		{
			var url = '../StringTie.servlet?ID='+App.getSessionId();
			url += '&cmd=GetAlignedSequencesForStringTie';
			url += '&items='+manuallySelected.join(',');
			Wizard.showLoadingAnimation('Loading bioassays...');
			Wizard.asyncJsonRequest(url, stringtie.manuallySelected);
		}
		else
		{
			Events.sendChangeEvent('alignedSequences');
		}
	}

	stringtie.manuallySelected = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var offset = frm.alignedSequences.length+1;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var as = alignedSequences[asNo];
				var option = stringtie.createListOption(asNo+offset, as, as.DO_NOT_USE == null);
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
			Events.sendChangeEvent('alignedSequences');
		}
	}

	stringtie.createListOption = function(index, alignedItem, selected)
	{
		var lib = alignedItem.lib;
		var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
		var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
		
		var tooltip = null;
		var name = (index) + ': ';
		if (libPlate) name += libPlate.name + ' - ';
		name += alignedItem.name;
		
		if (alignedItem.DO_NOT_USE)
		{
			name += ' [DoNotUse]';
			tooltip = 'DoNotUse-'+Strings.encodeTags(alignedItem.DO_NOT_USE+': '+alignedItem.DO_NOT_USE_COMMENT);
		}
		else if (alignedItem.AutoProcess == 'ReProcess')
		{
			name += ' [R]';
		}
		
		var option = new Option(name, alignedItem.id, false, selected);
		if (isYellow) option.className = 'yellow';
		if (tooltip) option.title = tooltip;
		option.alignedSequences = alignedItem;
		return option;
	}
	
	stringtie.alignedSequencesOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		selectionIsValid = false;
		var numSelected = 0;
		var numDoNotUse = 0;
		var invalidPipeline = null;
		Wizard.setInputStatus('alignedSequences');
		Wizard.hideGoNextConfirmation();

		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				numSelected++;
				var as = frm.alignedSequences[asNo].alignedSequences;
				if (as.DO_NOT_USE) numDoNotUse++;
				if (as.pipeline && as.pipeline != 'RNAseq/Hisat/StringTie')
				{
					invalidPipeline = Strings.encodeTags(as.name+' ('+as.pipeline)+') is not intended for the RNAseq/Hisat/StringTie pipeline.';
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'Select at least one item');
			return;
		}
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', invalidPipeline);
			return;
		}
		selectionIsValid = true;
		if (numDoNotUse > 0)
		{
			Wizard.setInputStatus('alignedSequences', 'warning', numDoNotUse + ' selected items marked as DoNotUse');
			Wizard.showGoNextConfirmation(true, 'Confirm ' + numDoNotUse + ' items marked as DoNotUse');
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'valid');
		}
	}

	// --- Step 2 -----------------------------------
	stringtie.initializeStep2 = function()
	{
		Wizard.setCurrentStep(2);
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		var frm = document.forms['reggie'];
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('FEATURE_EXTRACTION_PROTOCOL', 'stringtieProtocol', 'EXPRESSION_TYPE', 'StringTie');
		Reggie.loadSoftware('FEATURE_EXTRACTION_SOFTWARE', 'stringtieSoftware', 'EXPRESSION_TYPE,PARAMETER_SET', 'StringTie');
		
		// Load array design
		var url = '../StringTie.servlet?ID='+App.getSessionId() + '&cmd=GetArrayDesigns';
		Doc.addClass('arrayDesign', 'list-loading');
		frm.arrayDesign[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, stringtie.arrayDesignsLoaded);

		// Load clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=stringtie';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, stringtie.clustersLoaded);

		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	stringtie.validateStep2 = function(event)
	{
		if (!clusterIsValid) event.preventDefault();
		if (!arrayDesignIsValid) event.preventDefault();
	}
	
	stringtie.arrayDesignsLoaded = function(response)
	{
		Doc.removeClass('arrayDesign', 'list-loading');
		var frm = document.forms['reggie'];
		var designs = response.arrayDesigns;
		frm.arrayDesign.length = 0;
		for (var i = 0; i < designs.length; i++)
		{
			var design = designs[i];
			var option = new Option(design.name + ' ('+design.numFeatures+')', design.id, design.isDefault, design.isDefault);
			option.arrayDesign = design;
			frm.arrayDesign[frm.arrayDesign.length] = option;
		}
		if (frm.arrayDesign.length == 0)
		{
			Wizard.setInputStatus('arrayDesign', 'invalid', 'No available array designs');
			arrayDesignIsValid = false;
		}
		else
		{
			Wizard.setInputStatus('arrayDesign', 'valid');
			arrayDesignIsValid = true;
		}
	}
	
	stringtie.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.hosts;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available clusters');
			clusterIsValid = false;
		}
		else
		{
			stringtie.clusterOnChange();
		}
	}
	
	stringtie.softwareOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var item = target[target.selectedIndex].item;
		if (!item) item = {};
		
		Doc.element(target.id+'.parameterSet').innerHTML = Strings.encodeTags(item.ParameterSet || 'default');
		Doc.element(target.id+'.description').innerHTML = Strings.encodeTags(item.description);
		Wizard.setInputStatus(target.id, 'valid');

		if (item.ParameterSet)
		{
			var url = '../Install.servlet?ID='+App.getSessionId();
			url += '&cmd=GetParameterSetInfo';
			url += '&parameterSet='+encodeURIComponent(item.ParameterSet);
			url += '&targetId='+target.id;
			Wizard.asyncJsonRequest(url, stringtie.parameterSetInfoLoaded);
		}
	}
	
	stringtie.parameterSetInfoLoaded = function(response)
	{
		if (!response.parameters || response.parameters.length == 0)
		{
			Wizard.setInputStatus(response.targetId, 'warning', 'Can\'t find \'' + Strings.encodeTags(response.parameterSet) + '\' parameter set in reggie-config.xml');
		}
	}
	
	stringtie.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	stringtie.submit = function()
	{
		var frm = document.forms['reggie'];
			
		var submitInfo = {};
		submitInfo.stringtieSoftware = parseInt(frm.stringtieSoftware.value);
		submitInfo.stringtieProtocol = parseInt(frm.stringtieProtocol.value);
		submitInfo.arrayDesign = parseInt(frm.arrayDesign.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		submitInfo.autoConfirm = frm.autoConfirm.checked;
			
		var alignedSequences = [];
		submitInfo.alignedSequences = alignedSequences;
			
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				var as = {};
				as.id = frm.alignedSequences[asNo].alignedSequences.id;
				alignedSequences[alignedSequences.length] = as;
			}
		}

		var url = '../StringTie.servlet?ID='+App.getSessionId();
		url += '&cmd=StartStringTie';

		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, stringtie.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	stringtie.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return stringtie;
}();

Doc.onLoad(StringTie.initPage);

