var ViewVariants = function()
{
	var vgt = {};
	var debug = 0;
	
	var AA = [
		['A', /Ala/g], // Alanine 	GCA, GCC, GCG, GCT
		['B', /Asx/g], // Asparagine or Aspartic acid 	AAC, AAT, GAC, GAT
		['C', /Cys/g], // Cysteine 	TGC, TGT
		['D', /Asp/g], // Aspartic acid 	GAC, GAT
		['E', /Glu/g], // Glutamic acid 	GAA, GAG
		['F', /Phe/g], // Phenylalanine 	TTC, TTT
		['G', /Gly/g], // Glycine 	GGA, GGC, GGG, GGT
		['H', /His/g], // Histidine 	CAC, CAT
		['I', /Ile/g], // Isoleucine 	ATA, ATC, ATT
		['K', /Lys/g], // Lysine 	AAA, AAG
		['L', /Leu/g], // Leucine 	CTA, CTC, CTG, CTT, TTA, TTG
		['M', /Met/g], // Methionine 	ATG
		['N', /Asn/g], // Asparagine 	AAC, AAT
		['P', /Pro/g], // Proline 	CCA, CCC, CCG, CCT
		['Q', /Gln/g], // Glutamine 	CAA, CAG
		['R', /Arg/g], // Arginine 	AGA, AGG, CGA, CGC, CGG, CGT
		['S', /Ser/g], // Serine 	AGC, AGT, TCA, TCC, TCG, TCT
		['T', /Thr/g], // Threonine 	ACA, ACC, ACG, ACT
		['V', /Val/g], // Valine 	GTA, GTC, GTG, GTT
		['W', /Trp/g], // Tryptophan 	TGG
		['Y', /Tyr/g], // Tyrosine 	TAC, TAT
		['Z', /Glx/g], // Glutamine or Glutamic acid 	CAA, CAG, GAA, GAG
		['*', /Ter/g]  // Stop codon
	]

	
	vgt.initPage = function()
	{
		Buttons.addClickHandler('close', App.closeWindow);
		
		var url = '../VariantCalling.servlet?ID='+App.getSessionId();
		url += '&cmd=GetVcfStatistics';
		url += '&fileId='+Data.int('page-data', 'file-id');
		url += '&itemId='+Data.int('page-data', 'item-id');
		
		Wizard.showLoadingAnimation('Loading VCF file...');
		Wizard.asyncJsonRequest(url, vgt.onFilesLoaded);
	}
	
	vgt.onFilesLoaded = function(response)
	{
		var rba = response.rawbioassay;
		var snpData = response.snpData;
		var target = response.TargetedGenotype;
		
		var varSearchHome = Data.get('page-data', 'varsearch-home');
		
		var numIns = 0;
		var numDel = 0;
		var numSnv = 0;
		var numAlt = 0;
		
		var html = '';
		for (var snpNo = 0; snpNo < snpData.length; snpNo++)
		{
			var snp = snpData[snpNo].snp;
			var gt = snpData[snpNo].gt || { 'GT': '', 'AD': '' };;
			var info = snp.info;
			var cosmicId = info['cosmic_ID'];
			var dbSnpId = info['dbsnp_ID'];
			var type = info['TYPE'];
			var isAlt = gt.GT && gt.GT != '0/0';
			var isNoData = !gt.GT;
			if (isAlt) numAlt++;
			if (type == 'SNV') numSnv++;
			else if (type == 'Insertion') numIns++;
			else if (type == 'Deletion') numDel++;
			html += '<tr class="snp highlight'+(target && isAlt?' is-variant':'')+(isNoData?' no-data':'')+'">';
			html += '<td>'+Strings.encodeTags(snp.chromosome+':'+snp.position)+'</td>';
			html += '<td class="dottedleft ref">'+vgt.getRefAndAlt(snp.ref, snp.alt)+'</td>';
			html += '<td class="dottedleft">'+Strings.encodeTags(type)+'</td>';
			html += '<td class="dottedleft">'+vgt.getGenes(info['ncbiRefSeq'])+'</td>';
			html += '<td class="dottedleft gt">'+(gt.GT||'./.')+'</td>';
			html += '<td class="dottedleft ad">'+gt.AD+'</td>';
			html += '<td class="dottedleft">'+vgt.getHGVSc(info['ANN.HGVS.c[0]'] || info['cosmic_CDS'])+'</td>';
			html += '<td class="dottedleft">'+vgt.getHGVSp(info['ANN.HGVS.p[0]'] || info['cosmic_AA'])+'</td>';
			html += '<td class="dottedleft">'+vgt.getCosmicLink(cosmicId)+'</td>';
			html += '<td class="dottedleft">'+vgt.getDbSnpLink(dbSnpId)+'</td>';
			html += '<td class="zoom">';
			if (varSearchHome)
			{
				html += '<img src="'+varSearchHome+'/images/zoom.svg" class="link view-variant" data-line-no="'+snp.id+'" title="More information...">';
			}
			html += '</td>';
			html += '</tr>';
		}
		Doc.element('snp-list').innerHTML = html;
		Doc.show('snp-table', 'table');
		
		var zoom = document.getElementsByClassName('view-variant');
		for (var zoomNo = 0; zoomNo < zoom.length; zoomNo++)
		{
			Events.addEventHandler(zoom[zoomNo], 'click', vgt.viewVariantInVarSearch);
		}
		
		if (target)
		{
			Doc.element('targeted-genotype').innerHTML = Strings.encodeTags(target.description);
		}
		
		var summary = '';
		if (Data.get('page-data', 'mode')=='genotype')
		{
			summary = 'Found '+vgt.counted(numAlt, 'variant', 'variants')
				+ ' in '+vgt.counted(snpData.length, 'genotyped location', 'genotyped locations');
		}
		else
		{
			summary = snpData.length + ' passed filter ('
				+ numSnv + ' SNV; '
				+ vgt.counted(numIns, 'insertion', ' insertions')+'; '
				+ vgt.counted(numDel, 'deletion', ' deletions')+'); '
				+ vgt.counted(rba.VariantsRaw, 'raw variant', ' raw variants');			
		}
		Doc.element('snp-count').innerHTML = summary;
	}
	
	vgt.viewVariantInVarSearch = function(event)
	{
		var url = Data.get('page-data', 'varsearch-home')+'/hit-details.jsp';
		url += '?ID='+App.getSessionId();
		url += '&rbaId='+Data.int('page-data', 'item-id');
		url += '&fileId='+Data.int('page-data', 'file-id');
		
		url += '&lineNo='+Data.get(event.currentTarget, 'line-no');
		url += '&idx='+(Data.get('page-data', 'mode')=='genotype'?'targeted':'filtered');

		Dialogs.openPopup(url, 'VarSearchHit', 600, 400);
	}
	
	vgt.getAnnValues = function(info, key)
	{
		var list = [];
		var numAnnEntries = info['ANN#'];
		for (var i = 0; i < numAnnEntries; i++)
		{
			var val = info['ANN.'+key+'['+i+']'];
			if (val) list[list.length] = val;
		}
		
		return Strings.encodeTags(list.join(', '));
	}
	
	vgt.getGenes = function(genes)
	{
		if (!genes) return '';
		return Strings.encodeTags(genes.split(',').join(', '));
	}
	
	vgt.getHGVSc = function(hgvsc)
	{
		if (!hgvsc) return '';
		return hgvsc.replace(/c\./g, '').split(',').join(', ');
	}

	vgt.getHGVSp = function(hgvsp)
	{
		if (!hgvsp) return '';
		
		hgvsp = hgvsp.replace(/p\./g, ''); // Remove 'p.' prefix
		hgvsp = hgvsp.replace(/\?/g, ''); // Remove '?'
		
		for (var i = 0; i < AA.length; i++)
		{
			hgvsp = hgvsp.replace(AA[i][1], AA[i][0]);
		}
		
		var all = hgvsp.split(',');
		for (var i = 0; i < all.length; i++)
		{
			var aa = all[i];
			if (aa=='') 
			{
				all.splice(i, 1); // Skip empty entries
				i--;
			}
			else if (aa.length > 2 && aa.charAt(0)==aa.charAt(aa.length-1)) 
			{
				all[i] = aa.substring(0, aa.length-2)+'=';
			}
		}
		return all.join(', ');
	}

	
	vgt.getRefAndAlt = function(ref, alt)
	{
		if (ref.length > 6) ref = '<span title="'+ref+'">'+ref.substring(0, 5)+'...</span>';
		if (alt.length > 6) alt = '<span title="'+alt+'">'+alt.substring(0, 5)+'...</span>';
		return ref + ' › ' + alt;
	}
	
	vgt.getCosmicLink = function(cosmicId)
	{
		if (!cosmicId) return '';

		var tmp = cosmicId.split(',');
		var links = [];
		for (var i = 0; i < tmp.length; i++)
		{
			var link = '<a target="_blank" title="View in COSMIC (new window)"';
			link += ' href="https://cancer.sanger.ac.uk/cosmic/search?q='+encodeURIComponent(tmp[i])+'">';
			link += Strings.encodeTags(tmp[i])+'</a>';
			
			links[i] = link;
		}
		return links.join(', ');
	}
	
	vgt.getDbSnpLink = function(rsId)
	{
		if (!rsId) return '';

		var tmp = rsId.split(',');
		var links = [];
		for (var i = 0; i < tmp.length; i++)
		{
			var link = '<a target="_blank" title="View in dbSNP (new window)"';
			link += ' href="https://www.ncbi.nlm.nih.gov/snp/'+encodeURIComponent(tmp[i])+'">';
			link += Strings.encodeTags(tmp[i])+'</a>';
			
			links[i] = link;
		}
		return links.join(', ');
	}
	
	vgt.counted = function(n, one, many)
	{
		return n+' '+(n == 1 ? one : many);
	}
	
	return vgt;
}();

Doc.onLoad(ViewVariants.initPage);
