var Search = function()
{
	var search = {};
	var debug = 0;
	
	var flaggedRna;
	
	// Page initialization
	search.initPage = function()
	{
		// Step 1
		Events.addEventHandler('labelFilter', 'change', search.filterOnChange);
		Events.addEventHandler('flagFilter', 'change', search.filterOnChange);
		Events.addEventHandler('rnaOriginalFilter', 'change', search.filterOnChange);
		Events.addEventHandler('rnaRemainFilter', 'change', search.filterOnChange);
		Events.addEventHandler('rqsRinFilter', 'change', search.filterOnChange);
		Events.addEventHandler('lysateRemainFilter', 'change', search.filterOnChange);
		Events.addEventHandler('lysateUsedQuantityFilter', 'change', search.filterOnChange);
		Events.addEventHandler('specimenRemainFilter', 'change', search.filterOnChange);
		Events.addEventHandler('libExistsFilter', 'change', search.filterOnChange);
		Events.addEventHandler('rawDataExistsFilter', 'change', search.filterOnChange);
		
		Buttons.addClickHandler('btnReExtractSpecimen', search.reExtractSpecimen);
		Buttons.addClickHandler('btnReExtractLysate', search.reExtractLysate);
		Buttons.addClickHandler('btnFlagSpecimen', search.flagSpecimen);
		Buttons.addClickHandler('btnReProcessRna', search.reProcessRna);
		Buttons.addClickHandler('btnUnflagRna', search.unflagRna);
		Buttons.addClickHandler('btnNewBioMaterialList', search.newBioMaterialList);
		Buttons.addClickHandler('check_uncheck', search.selectAll);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', search.submit);

		var url = '../FlaggedRna.servlet?ID='+App.getSessionId();
		url += '&cmd=LoadFlaggedRna';
		Wizard.showLoadingAnimation('Loading flagged RNA...');
		Wizard.asyncJsonRequest(url, search.onRnaLoaded);
	}
	
	search.onRnaLoaded = function(response)
	{
		flaggedRna = response.rna;
		if (flaggedRna.length == 0)
		{
			Wizard.setFatalError('No flagged RNA available for processing.');
			return;
		}
		var frm = document.forms['reggie'];
		
		var yellowImg = '<img src="../images/yellow-label.png">';
		var html = '';
		var usedFlags = [];
		for (var rnaNo = 0; rnaNo < flaggedRna.length; rnaNo++)
		{
			var rna = flaggedRna[rnaNo];
			rna.include = true;
			if (rna.flag && usedFlags.indexOf(rna.flag) == -1)
			{
				usedFlags[usedFlags.length] = rna.flag;
			}
			var qs = rna.rin || rna.rqs;
			
			var isYellow = rna.specimen && rna.specimen.YellowLabel != null;
			var img = isYellow ? yellowImg : '';
			
			var stringTieCount = search.countRawBioAssaysByType(rna.rawbioassays, 'variant.sequencing.stringtie');
			var cufflinksCount = search.countRawBioAssaysByType(rna.rawbioassays, 'cufflinks');
			var variantCallCount = search.countRawBioAssaysByType(rna.rawbioassays, 'variant.sequencing.variant-call');
			
			html += '<tr class="highlight ' + (rnaNo % 2 == 0 ? 'oddrow' : 'evenrow') + (isYellow ? ' yellow-specimen' : '') + (rna.DO_NOT_USE ? ' donotuse' : '') + '" id="rna.'+rna.id + '">';
			html += '<td><span id="index.'+rna.id+'">'+(rnaNo+1)+'</span> <input type="checkbox" name="rna.'+rna.id+'">';
			html += '<img class="link case-summary" data-name="'+Strings.encodeTags(rna.name)+'" src="../images/case_summary.png"></td>';
			html += '<td class="iconcol-1">'+search.doNotUseWarning(rna)+'</td>';
			html += '<td class="dottedleft if-yellow">'+img+Strings.encodeTags(rna.name)+'</td>';
			html += '<td class="if-yellow">'+Strings.encodeTags(rna.label)+'</td>';
			html += '<td>'+Strings.encodeTags(rna.flag) + '</td>';
			html += '<td>'+Reggie.formatNumber(rna.originalQuantity, null, 2) + '</td>';
			html += '<td>'+Reggie.formatNumber(rna.remainingQuantity, null, 2) + '</td>';
			html += '<td>'+(qs == -100 ? '-100' : Reggie.formatNumber(qs, null, 2))+'</td>';
			
			html += '<td class="dottedleft">'+Math.round(rna.lysate.remainingQuantity) + '</td>';
			html += '<td>'+Reggie.formatNumber(rna.lysate.usedQuantity/1000, null, 1) + '</td>';
			html += '<td class="dottedleft">'+Reggie.formatNumber(rna.specimen.remainingQuantity/1000, null, 1) + '</td>';
			html += '<td>'+Reggie.reformatDate(rna.specimen.registered)+'</td>';
			html += '<td class="dottedleft">'+(rna.libraries.length > 0 ? rna.libraries.length : '-')+'</td>';
			html += '<td class="dottedleft">'+(rna.rawbioassays.length > 0 ? stringTieCount +' / '+cufflinksCount+' / '+variantCallCount : '-')+'</td>';
			html += '</tr>';
		}
		
		usedFlags.sort();
		for (var flagNo = 0; flagNo < usedFlags.length; flagNo++)
		{
			var flag = usedFlags[flagNo];
			frm.flagFilter[frm.flagFilter.length] = new Option(flag);
		}
		
		Doc.element('rna-list').innerHTML = html;
		Doc.element('num-matching').innerHTML = flaggedRna.length;
		
		Doc.show('step-1');

		var cs = document.getElementsByClassName('case-summary');
		for (var i = 0; i < cs.length; i++)
		{
			Events.addEventHandler(cs[i], 'click', Reggie.openCaseSummaryPopupOnEvent);
		}
	}
	
	search.countRawBioAssaysByType = function(rawbioassays, rawDataType)
	{
		var count = 0;
		for (var i = 0; i < rawbioassays.length; i++)
		{
			if (rawbioassays[i].rawDataType == rawDataType) count++;
		}
		return count;
	}
	
	search.doNotUseWarning = function(rna)
	{
		var warning = '';
		if (rna && rna.DO_NOT_USE)
		{
			warning += '<img src="../images/warning.png" title="DoNotUse-'+Strings.encodeTags(rna.DO_NOT_USE + ': ' + rna.DO_NOT_USE_COMMENT)+'">';
		}
		return warning;
	}

	
	search.selectAll = function()
	{
		var frm = document.forms['reggie'];
		var firstChecked = null;
		for (var rnaNo = 0; rnaNo < flaggedRna.length; rnaNo++)
		{
			var rna = flaggedRna[rnaNo];
			if (rna.include)
			{
				if (firstChecked == null)
				{
					firstChecked = frm['rna.'+rna.id].checked;
				}
				frm['rna.'+rna.id].checked = !firstChecked;
			}
		}
	}
	
	search.filterOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		var filter = new Filter();
		// RNA filters
		filter.addTextCriteria(RnaProperty.LabelExists, frm.labelFilter.value);
		filter.addTextCriteria(RnaProperty.Flag, frm.flagFilter.value);
		filter.addNumericCriteria(RnaProperty.OriginalQuantity, frm.rnaOriginalFilter.value);
		filter.addNumericCriteria(RnaProperty.RemainingQuantity, frm.rnaRemainFilter.value);
		filter.addNumericCriteria(RnaProperty.RqsOrRin, frm.rqsRinFilter.value);
		// Lysate filters
		filter.addNumericCriteria(RnaProperty.LysateRemaining, frm.lysateRemainFilter.value);
		filter.addNumericCriteria(RnaProperty.LysateUsedQuantity, frm.lysateUsedQuantityFilter.value, 1000);
		// Specimen filters
		filter.addNumericCriteria(RnaProperty.SpecimenRemaining, frm.specimenRemainFilter.value, 1000);
		// Lib and raw data
		filter.addNumericCriteria(RnaProperty.NumLibraries, frm.libExistsFilter.value);
		filter.addNumericCriteria(RnaProperty.NumRawBioAssays, frm.rawDataExistsFilter.value);

		var index = 0;
		for (var rnaNo = 0; rnaNo < flaggedRna.length; rnaNo++)
		{
			var rna = flaggedRna[rnaNo];
			
			rna.include = filter.check(rna);
			if (rna.include)
			{
				index++;
				var evenRow = index % 2 == 0;
				var tr = Doc.element('rna.'+rna.id);
				Doc.addOrRemoveClass(tr, 'oddrow', !evenRow);
				Doc.addOrRemoveClass(tr, 'evenrow', evenRow);
				Doc.element('index.'+rna.id).innerHTML = index;
				Doc.show(tr);
			}
			else
			{
				Doc.hide('rna.'+rna.id);
			}
		}
		
		Doc.element('num-matching').innerHTML = index;
	}
	
	search.checkSelected = function(noNotification)
	{
		var frm = document.forms['reggie'];
		var selected = [];
		
		for (var rnaNo = 0; rnaNo < flaggedRna.length; rnaNo++)
		{
			var rna = flaggedRna[rnaNo];
			if (rna.include && frm['rna.'+rna.id].checked)
			{
				selected[selected.length] = rna.id;
			}
		}
		
		if (selected.length == 0 && !noNotification)
		{
			Forms.showNotification('check_uncheck', 'Please select at least one RNA from the list', null, 'pointer-below');
		}
		
		return selected;
	}
	
	search.reExtractSpecimen = function()
	{
		var frm = document.forms['reggie'];
		var selected = search.checkSelected();
		if (selected.length == 0) return;
		
		if (!confirm('This will remove ' + selected.length + ' RNA items from this list and schedule re-extraction from specimen.\n\nContinue?'))
		{
			return;
		}
		
		submitInfo = {};
		submitInfo.rna = selected;
		
		submitUrl = '../FlaggedRna.servlet?ID='+App.getSessionId();
		submitUrl += '&cmd=ReExtractFromSpecimen';
		Wizard.showLoadingAnimation('Registering selected RNA for re-extraction from specimen...');
		Wizard.goRegister();
	}
	
	search.reExtractLysate = function()
	{
		var frm = document.forms['reggie'];
		var selected = search.checkSelected();
		if (selected.length == 0) return;
		
		if (!confirm('This will remove ' + selected.length + ' RNA items from this list and schedule re-extraction from lysate.\n\nContinue?'))
		{
			return;
		}
		
		submitInfo = {};
		submitInfo.rna = selected;
		
		submitUrl = '../FlaggedRna.servlet?ID='+App.getSessionId();
		submitUrl += '&cmd=ReExtractFromLysate';
		Wizard.showLoadingAnimation('Registering selected RNA for re-extraction from lysate...');
		Wizard.goRegister();
	}

	
	search.flagSpecimen = function()
	{
		var frm = document.forms['reggie'];
		var selected = search.checkSelected();
		if (selected.length == 0) return;
		
		if (!confirm('This will remove ' + selected.length + ' RNA items from this list and flag the specimen instead.\n\nContinue?'))
		{
			return;
		}
		
		submitInfo = {};
		submitInfo.rna = selected;
		
		submitUrl = '../FlaggedRna.servlet?ID='+App.getSessionId();
		submitUrl += '&cmd=FlagSpecimen';
		Wizard.showLoadingAnimation('Flagging specimen for the selected RNA...');
		Wizard.goRegister();
	}
	
	search.reProcessRna = function()
	{
		var frm = document.forms['reggie'];
		var selected = search.checkSelected();
		if (selected.length == 0) return;
		
		if (!confirm('This will schedule ' + selected.length + ' RNA items for library preparation.\n\nContinue?'))
		{
			return;
		}
		
		submitInfo = {};
		submitInfo.rna = selected;
		
		submitUrl = '../FlaggedRna.servlet?ID='+App.getSessionId();
		submitUrl += '&cmd=ReProcessRna';
		Wizard.showLoadingAnimation('Registering selected RNA for new library preparation...');
		Wizard.goRegister();
	}
	
	search.unflagRna = function()
	{
		var frm = document.forms['reggie'];
		var selected = search.checkSelected();
		if (selected.length == 0) return;
		
		if (!confirm('This will remove ' + selected.length + ' RNA items from this list and reset their flags.\n\nContinue?'))
		{
			return;
		}
		
		submitInfo = {};
		submitInfo.rna = selected;
		
		submitUrl = '../FlaggedRna.servlet?ID='+App.getSessionId();
		submitUrl += '&cmd=UnflagRna';
		Wizard.showLoadingAnimation('Unflagging selected RNA...');
		Wizard.goRegister();
	}
	
	search.newBioMaterialList = function()
	{
		var frm = document.forms['reggie'];
		var selected = search.checkSelected();
		if (selected.length == 0) return;

		var url = 'new_list.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'NewBioMaterialList', 600, 400);
	}

	var submitInfo;
	var submitUrl;

	search.submit = function()
	{
		Wizard.asyncJsonRequest(submitUrl, search.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	search.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	return search;
}();

Doc.onLoad(Search.initPage);


/**
	Holds one or more filter criterias. Use the 'add*' methods
	to add criterias to the filter, and the 'check(rna)' method
	to evaluate all criterias against a single RNA item.
*/
var Filter = function()
{
	this.criterias = [];
}

/**
	Adds a text criteria comparing the given RNA property to
	a static value (f2).
*/
Filter.prototype.addTextCriteria = function(rnaProp, f2)
{
	if (f2 != '') 
	{
		this.criterias[this.criterias.length] = function(rna) { return Comparator.eq(rnaProp(rna), f2) };
	}
}

/**
	Adds a numeric criteria comparing the given RNA property to
	a static value (f2). The 'f2' value may start with either '<' or '>'
	to compare with less than/greater than operators, otherwise
	it will use the 'approximately equals' operator.
*/
Filter.prototype.addNumericCriteria = function(rnaProp, f2, factor)
{
	if (f2 != '')
	{
		var firstChar = f2.charAt(0);
		if (!firstChar.match(/[\d\-]/)) f2 = f2.substring(1);

		var cmp = Comparator.get(firstChar, Comparator.approx);
		var limit = parseFloat(f2);
		if (factor) limit *= factor;

		this.criterias[this.criterias.length] = function(rna) { return cmp(rnaProp(rna), limit) };
	}
}

/**
	Evaluate all criterias. Return true if all pass, false otherwise.
*/
Filter.prototype.check = function(rna)
{
	for (var i = 0; i < this.criterias.length; i++)
	{
		if (!this.criterias[i](rna)) return false;
	}
	return true;
}

/**
	Functions for extracting various properties from
	RNA items. Each method accepts a single RNA as a parameter
	and return a value from the RNA item. The methods
	are used as parameters to Filter.add*Criteria() methods.
*/
var RnaProperty = function()
{
	var p = {};
	
	// Get "1" if a rna.label exists, or "0" if not
	p.LabelExists = function(rna)
	{
		return rna.label ? '1' : '0';
	}
	
	// Get the rna.flag value
	p.Flag = function(rna)
	{
		return rna.flag;
	}
	
	// Get the rna.originalQuantity value
	p.OriginalQuantity = function(rna)
	{
		return rna.originalQuantity;
	}
	
	// Get the rna.remainingQuantity value
	p.RemainingQuantity = function(rna)
	{
		return rna.remainingQuantity;
	}
	
	// Get the rna.rqs or rna.rin value
	p.RqsOrRin = function(rna)
	{
		return rna.rqs || rna.rin;
	}
	
	// Get the rna.lysate.remainingQuantity
	p.LysateRemaining = function(rna)
	{
		return rna.lysate.remainingQuantity;
	}
	
	// Get the rna.lysate.usedQuantity
	p.LysateUsedQuantity = function(rna)
	{
		return rna.lysate.usedQuantity;
	}
	
	// Get the rna.specimen.remainingQuantity
	p.SpecimenRemaining = function(rna)
	{
		return rna.specimen.remainingQuantity;
	}
	
	// Get the number of rna.libraries
	p.NumLibraries = function(rna)
	{
		return rna.libraries.length;
	}

	// Get the number of rna.rawbioassays
	p.NumRawBioAssays = function(rna)
	{
		return rna.rawbioassays.length;
	}

	return p;
}();

/**
	Comparator functions for comparing two values.
*/
var Comparator = function()
{
	var cmp = {};
	
	/**
		Return a comparator based on the symbol
		which can be either '>' or '<' to return
		Comparator.gt or Comparator.lt. Otherwise
		the given default or Comparator.eq is returned.
	*/
	cmp.get = function(symbol, defaultCmp)
	{
		var c;
		if (symbol == '<')
		{
			c = cmp.lt;
		}
		else if (symbol == '>')
		{
			c = cmp.gt;
		}
		else
		{
			c = defaultCmp || cmp.eq;
		}
		return c;
	}
	
	// Compare if two values are equal
	cmp.eq = function(v1, v2)
	{
		return v1 == v2;
	}
	
	// Compare if two values are appoximately equals (difference is not more than 0.05)
	cmp.approx = function(v1, v2)
	{
		return Math.abs(v2 - v1) < 0.05;
	}
	
	// Compare if v1 is greater than or equal to v2
	cmp.gt = function(v1, v2)
	{
		return v1 >= v2;
	}
	
	// Compare if v1 is less than or equal to v2
	cmp.lt = function(v1, v2)
	{
		return v1 <= v2;
	}
	
	return cmp;
}();

