var Align = function()
{
	var align = {};
	var debug = 0;
	
	var JOB_IMAGE = { 'DONE': 'ok.png', 'ERROR': 'error.png'};
	
	var alignedSequences;
	
	// Page initialization
	align.initPage = function()
	{
		
		// Step 1

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', align.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmedAlignedSequences';
		Wizard.asyncJsonRequest(url, align.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	align.initializeStep1 = function(response)
	{
		alignedSequences = response.alignedSequences;
		if (alignedSequences.length == 0)
		{
			Wizard.setFatalError('No aligned sequences available for processing.');
			return;
		}
		
		var html = '<table id="alignedSequencesTable">';
		html += '<thead class="bg-filled-100">';
		// Header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th></th>';
		html += '<th></th>';
		html += '<th class="dottedleft" colspan="4">Reads</th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th class="dottedleft" colspan="3">Actions</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		
		html += '<tr>';
		html += '<th>Plate</th>';
		html += '<th class="dottedleft">Library</th>';
		html += '<th>MIPS_Panel</th>';
		html += '<th>Barcode</th>';
		html += '<th class="dottedleft">PF_READS</th>';
		html += '<th>ALIGNED_PAIRS</th>';
		html += '<th class="icon-col"></th>';
		html += '<th>Duplicates</th>';
		html += '<th class="dottedleft">Job</th>';
		html += '<th class="dottedleft">Files</th>';
		html += '<th class="dottedleft">Flag RNA</th>';
		html += '<th>Re-align</th>';
		html += '<th>TODO</th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		var numWithError = 0;
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];

			var lib = aligned.lib;
			lib.stratagene = Reggie.isStratagene(lib.name);
			lib.external = Reggie.isExternal(lib.name);
			var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
			
			var merged = aligned.merged;
			
			var pfReads = Reggie.formatCount(merged.PF_READS, '-');
			var alignedReads = Reggie.formatCount(aligned.ALIGNED_PAIRS, '-');
			var alignedPercent = aligned.ALIGNED_PAIRS && merged.PF_READS ? ' (' + Math.round(100*aligned.ALIGNED_PAIRS/merged.PF_READS) + '%)' : '';
			var percentDuplicates = aligned.FRACTION_DUPLICATION == null ? '-' : Math.round(100*aligned.FRACTION_DUPLICATION)+'%';
			var job = aligned.job;
			
			var flag = null;
			var isError = job.status == 'ERROR';
			
			html += '<tr class="highlight ' + (alignedNo % 4 < 2 ? 'evenrow' : 'oddrow') + '">';
			html += '<td>'+(libPlate ? Strings.encodeTags(libPlate.name+' '+lib.bioWell.location) : '-')+'</td>';
			html += '<td class="dottedleft">'+Strings.encodeTags(aligned.name)+'</td>';
			html += '<td>'+Strings.encodeTags(lib.MIPSPanel)+'</td>';
			html += '<td>'+Strings.encodeTags(lib.barcode.name)+'</td>';
			html += '<td class="dottedleft">'+pfReads+'</td>';
			html += '<td>'+alignedReads+alignedPercent+'</td>';
			html += '<td class="icon-col">';
			html += '</td>';

			html += '<td>'+percentDuplicates+'</td>';
			html += '<td class="dottedleft">';
			html += '<span class="link item-link" data-item-type="JOB" data-item-id="'+job.id+'" title="'+Strings.encodeTags(job.statusMessage)+'"><img src="../images/'+JOB_IMAGE[job.status]+'"></span>';
			html += '</td>';
			if (aligned.DataFilesFolder && job.server)
			{
				html += '<td class="dottedleft">';
				html += '<span class="link file-link" data-server="'+Strings.encodeTags(job.server)+'" data-folder="'+Strings.encodeTags(aligned.DataFilesFolder)+'" data-item-id="'+aligned.id+'" title="View generated files"><img src="../images/remote_folder.png"></span>';
				html += '</td>';
			}
			else
			{
				html += '<td class="dottedleft">-</td>';
			}
			
			var realignChecked = isError ? ' checked' : '';
			if (isError) numWithError++;
			if (lib.stratagene || lib.external)
			{
				html += '<td class="dottedleft">-</td>';
			}
			else
			{
				html += '<td class="dottedleft"><input type="checkbox" name="flag.'+aligned.id+'"'+(flag != null ? ' checked value="'+flag+'"':'')+'></td>';
			}
			
			html += '<td><input type="checkbox" name="realign.'+aligned.id+'"'+realignChecked+'></td>';
			html += '<td><input type="checkbox" name="nextStepTODO.'+aligned.id+'"></td>';
			html += '<td class="dottedleft comment"><input type="text" name="comment.'+aligned.id+'"></td>';
			html += '</tr>';

		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('alignedSequences').innerHTML = html;
		if (numWithError > 0) Doc.show('delete-items', 'table');
		
		// Add click handler to items
		var clickableItems = document.getElementsByClassName('item-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		// Add click handler to file links
		var clickableItems = document.getElementsByClassName('file-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', align.fileLinkOnClick);
		}

		Doc.show('step-1');
		Doc.show('goregister');
	}
	
	align.fileLinkOnClick = function(event)
	{
		var folder = Data.get(event.currentTarget, 'folder');
		var server = Data.get(event.currentTarget, 'server');
		var itemId = Data.get(event.currentTarget, 'item-id');
		server = server.replace(/\s*\[.*\]/, '');
		var url = '../analysis/view_remote_files.jsp?ID=' + App.getSessionId();
		url += '&clusterId='+server;
		url += '&path='+encodeURIComponent(folder);
		url += '&itemType=DERIVEDBIOASSAY&itemId='+itemId;
		Dialogs.openPopup(url, 'ViewProjectFiles', 750, 500);
	}
	
	align.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.deleteItemsCreatedByFailedJobs = frm.deleteItemsCreatedByFailedJobs.checked;
		
		var all = [];
		submitInfo.alignedSequences = all;
		for (var alignedNo = 0; alignedNo < alignedSequences.length; alignedNo++)
		{
			var aligned = alignedSequences[alignedNo];
			var tmp = {};
			tmp.id = aligned.id;
			
			tmp.flag = frm['flag.'+aligned.id] && frm['flag.'+aligned.id].checked ? frm['flag.'+aligned.id].value : null;
			tmp.realign = frm['realign.'+aligned.id].checked;
			tmp.nextStepTODO = frm['nextStepTODO.'+aligned.id].checked;
			tmp.comment = frm['comment.'+aligned.id].value;
			
			all[all.length] = tmp;
		}
		
		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterAlignment';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, align.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	align.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}
	
	
	return align;
}();

Doc.onLoad(Align.initPage);

