var FlowCell = function()
{
	var flowcell = {};
	var debug = 0;
	
	var COLORS = ['blue', 'green', 'yellow', 'red'];
	var nextColor = 0;

	// Page initialization
	flowcell.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);		
			var idList = Data.get('page-data', 'flowcells');
			var url = '../FlowCell.servlet?ID='+App.getSessionId();
			url += '&cmd=GetFlowCellInfo';
			url += '&idlist='+idList;
			Wizard.showLoadingAnimation('Loading flow cell info...');
			Wizard.asyncJsonRequest(url, flowcell.initializeProtocol);
		}
		else
		{
			Buttons.addClickHandler('gocreate', flowcell.createProtocol);
			var url = '../FlowCell.servlet?ID='+App.getSessionId();
			url += '&cmd=GetUnprocessedFlowCells&pipeline=MIPS';		
			Wizard.showLoadingAnimation('Loading flow cells...');
			Wizard.asyncJsonRequest(url, flowcell.initializeStep1);
		}
	}


	flowcell.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var flowCells = response.flowCells;
		
		if (flowCells.length > 0)
		{
			var lastOption = null;
			for (var fcNo=0; fcNo < flowCells.length; fcNo++)
			{
				var fc = flowCells[fcNo];
				var option = flowcell.getOptionForFlowCell(fc);
				if (fcNo == 0 || (lastOption != null && lastOption.selected && lastOption.title == option.title)) 
				{
					// Select the first option and more if they have the same pool(s)
					option.selected = true;
				}
				frm.flowcells[frm.flowcells.length] = option;
				lastOption = option;
			}
		}
		else
		{
			Wizard.setFatalError('No flow cells available for processing.');
			return;
		}

		// All is ok
		Doc.show('step-1');
		Doc.show('gocreate')
		frm.flowcells.focus();
	}
	
	
	flowcell.getOptionForFlowCell = function(fc)
	{
		var name = fc.name +': ';

		var numPools = fc.pools.length;
		var firstPoolNum = flowcell.getPoolNum(fc.pools[0].name);
		var lastPoolNum = flowcell.getPoolNum(fc.pools[fc.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += fc.pools[0].name + ' — ' + fc.pools[fc.pools.length-1].name;
			}
			else
			{
				name += fc.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += fc.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}
		
		name += ' [';
		if (fc.FlowCellType) name += fc.FlowCellType + '; ';
		name += fc.pipeline + ']';

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = fc.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}
		
		var option = new Option(name, fc.id);
		option.title = title;
		option.flowCell = fc;

		return option;
	}
	
	flowcell.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;	
	}


	flowcell.createProtocol = function()
	{
		var frm = document.forms['reggie'];
		
		var selected = [];
		for (var i = 0; i < frm.flowcells.length; i++)
		{
			if (frm.flowcells[i].selected) 
			{
				selected[selected.length] = frm.flowcells[i].value;
			}
		}
		
		if (selected.length == 0) 
		{
			Wizard.setInputStatus('flowcells', 'invalid', 'Select at least one flow cell');
			Wizard.notifyError();
			return;
		}

		frm.submit();
	}
	
	flowcell.initializeProtocol = function(response)
	{
		var flowCells = response.flowCells;

		var allColors = [];
		var useDarker = false;

		var names = [];
		
		for (var fcNo = 0; fcNo < flowCells.length; fcNo++)
		{
			var fc = flowCells[fcNo];
			names[names.length] = fc.name;
			
			var poolColors = [];
			var poolsRow = Doc.element('all-pools-'+fcNo);

			Doc.element('flowCellName.'+fcNo).innerHTML = Strings.encodeTags(fc.name);
			Doc.element('flowCellDescription.'+fcNo).innerHTML = Strings.encodeTags(fc.description);
			Doc.element('sequencingCycles.'+fcNo).innerHTML = Strings.encodeTags(fc.SequencingCycles);
			Doc.element('flowCellId.'+fcNo).innerHTML = Strings.encodeTags(fc.externalId);
			Doc.element('flowCellType.'+fcNo).innerHTML = Strings.encodeTags(fc.FlowCellType || '') + '; ' + Strings.encodeTags(fc.pipeline || '');
			
			if (fc.FlowCellType == 'NextSeq') Doc.hide('pos.b.'+fcNo);
			
			var lanesHtml = '';
			for (var laneNo = 0; laneNo < fc.numLanes; laneNo++)
			{
				var pool = fc.lanes[laneNo];
				var poolName = pool.name;
				var color = poolColors[poolName];
				if (!color)
				{
					color = allColors[poolName];
					if (!color)
					{
						color = COLORS[nextColor] + (useDarker ? ' darker' : '');
						allColors[poolName] = color;
						nextColor++;
						if (nextColor == COLORS.length)
						{
							nextColor = 0;
							useDarker = !useDarker;
						}
					}
					poolColors[poolName] = color;
					var td = document.createElement('td');
					td.className = 'pool-legend ' + color;
					td.innerHTML = poolName;
					poolsRow.appendChild(td);
				}

				lanesHtml += '<tr>';
				lanesHtml += '<td class="lane-no">'+(laneNo+1)+'</td>';
				lanesHtml += '<td class="pool-name '+color+'">'+Strings.encodeTags(poolName)+'</td>';
				lanesHtml += '<td class="remarks">';
				lanesHtml += '</td>';
				lanesHtml += '</tr>';
			}
			
			Doc.element('pools.'+fcNo).innerHTML = lanesHtml;
				
			if (poolColors.length <= 6)
			{
				// White-space at end to avoid pools getting to wide
				var td = document.createElement('td');
				td.innerHTML = '';
				poolsRow.appendChild(td);
			}
		}
		
		document.title = 'Lab protocol for ' + names.join(', ');
		Doc.show('all-protocol');
	}
	
	return flowcell;
}();

Doc.onLoad(FlowCell.initPage);

