var ImportPlates = function()
{
	var mips = {};
	var debug = 0;
	
	var prefixIsValid = false;
	
	var importData;
	
	// Page initialization
	mips.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', mips.validateStep1);

		Events.addEventHandler('prefix', 'change', mips.prefixOnChange);
		Events.addEventHandler('prefix', 'keypress', Wizard.focusOnEnter);
		Wizard.initFileSelectionField('importfile');
		Events.addEventHandler('importfile', 'change', mips.importFileOnChange);
		Events.addEventHandler('sheet', 'change', mips.worksheetOnChange);
		
		Events.addEventHandler('plateNames', 'change', mips.displayAliquotsOnPlate);
		
		Events.addEventHandler('pool-row', 'click', mips.togglePool);
		Events.addEventHandler('pool-row', 'mouseover', mips.highlightPool);
		Events.addEventHandler('pool-row', 'mouseout', mips.highlightPool);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', mips.submit);

		Plate.init(8, 12, WellPainter);
		Plate.paint(Plate.getWells());
		window.addEventListener('beforeunload', mips.confirmLeaveWizard);
		Wizard.keepSessionAlive();
		
		Doc.show('step-1');
		Doc.show('gocancel');
		Doc.show('goregister');
		Events.sendChangeEvent('prefix');
	}
	
	/**
		Ask for confirmation to abort if DNA has been placed on the plate.
	*/
	mips.confirmLeaveWizard = function(event)
	{
		if (Plate.getUsed().length > 0)
		{
			event.preventDefault();
		}
	}

	mips.prefixOnChange = function()
	{
		var frm = document.forms['reggie'];
		Wizard.setInputStatus('prefix');
		
		var prefix = frm.prefix.value.toUpperCase().replace(/_/g, '');
		prefixIsValid = false;
		if (prefix.length < 2)
		{
			Wizard.setInputStatus('prefix', 'invalid', 'The prefix should have at least 2 characters.');
			frm.prefix.focus();
			return;
		}
		
		frm.prefix.value = prefix+'_';
		Wizard.setInputStatus('prefix', 'valid');
		prefixIsValid = true;
		
		mips.prevalidateImportFile();
	}
	
	mips.importFileOnChange = function()
	{
		Doc.hide('worksheetSection');
		Doc.hide('worksheetSection.help');
		mips.prevalidateImportFile();
	}
	
	mips.worksheetOnChange = function()
	{
		mips.prevalidateImportFile();
	}
	
	mips.validateStep1 = function(event)
	{
		if (!prefixIsValid || !Wizard.isValid('importfile'))
		{
			event.preventDefault();
		}
	}

	mips.prevalidateImportFile = function()
	{
		Doc.hide('platePreview');
		Doc.hide('pool-row');
		Wizard.hideWizardStatus();
		Wizard.hideGoNextConfirmation();
		
		var frm = document.forms['reggie'];
		var path = frm.importfile.value;
		if (!prefixIsValid || path == '') return;
		
		Wizard.setInputStatus('importfile');
		Data.set('importfile', 'valid', 0);

		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=PreValidateImportFile';
		url += '&path='+encodeURIComponent(path);
		url += '&prefix='+encodeURIComponent(frm.prefix.value);
		url += '&sheet='+encodeURIComponent(frm.sheet.value);
		
		Wizard.showLoadingAnimation('Validating ' + Strings.encodeTags(path));
		Wizard.asyncJsonRequest(url, mips.onImportFileValidated, 'POST');
	}

	mips.onImportFileValidated = function(response)
	{
		var frm = document.forms['reggie'];

		Wizard.setInputStatus('importfile');
		Wizard.showFinalMessage(response.messages);
		
		importData = response.importFile;

		var sheetNames = importData.sheetNames;
		if (sheetNames && sheetNames.length)
		{
			var currentSheet = frm.sheet.value;
			frm.sheet.length = 0;
			for (var sNo = 0; sNo < sheetNames.length; sNo++)
			{
				var sheet = sheetNames[sNo];
				frm.sheet[frm.sheet.length] = new Option(sheet, sheet, false, currentSheet == sheet);
			}
			if (frm.sheet.length > 1) 
			{
				Doc.show('worksheetSection');
				Doc.show('worksheetSection.help');
			}
		}
		
		var plates = importData.plates;
		frm.plateNames.length = 0;
		
		if (plates && plates.length)
		{
			for (var pNo = 0; pNo < plates.length; pNo++)
			{
				var plate = plates[pNo];
				var text = plate.name;
				if (plate.error) 
				{
					text += ' (' + plate.error + ')';
				}
				else
				{
					text += ' (' + plate.aliquots.length + ' aliquots)';
				}
				frm.plateNames[frm.plateNames.length] = new Option(text);
			}
		}
		
		if (importData.valid)
		{
			Wizard.setInputStatus('importfile', importData.warnings > 0 ? 'warning' : 'valid');
			Data.set('importfile', 'valid', 1);
			if (importData.warnings > 0)
			{
				Wizard.showGoNextConfirmation(true, 'Check to verify registration of plate with warnings.');
			}
		}
		else
		{
			Wizard.setInputStatus('importfile', 'invalid');
		}
		
		mips.displayAliquotsOnPlate();
	}

	mips.displayAliquotsOnPlate = function()
	{
		Doc.element('filename').innerHTML = Strings.encodeTags(importData.filename);
		Doc.element('numPlates').innerHTML = importData.numPlates;
		Doc.element('numAliquots').innerHTML = importData.numAliquots;
		Doc.element('numErrors').innerHTML = importData.errors;
		Doc.element('numWarnings').innerHTML = importData.warnings;
		
		Doc.element('importPossible').innerHTML = importData.valid ? '<img src="../images/ok.png">Import is possible!' : '<img src="../images/error.png">Import is not possible!';
		
		Doc.show('platePreview');
		var frm = document.forms['reggie'];
		var plateIndex = frm.plateNames.selectedIndex;
		
		var wells = Plate.getWells();
		Plate.clear(wells);
		MipsColor.reset();
		COLUMN_CLASSES = new Array(Plate.columns);
		
		if (plateIndex >= 0 && importData.plates.length > plateIndex)
		{
			var plate = importData.plates[plateIndex];
			var aliquots = plate.aliquots;
			for (var aNo = 0; aNo < aliquots.length; aNo++)
			{
				var aliquot = aliquots[aNo];
				var well = Plate.getWellByLocation(aliquot.location);
				if (well && !well.extract) 
				{
					if (plate.error && !aliquot.error) aliquot.error = plate.error;
					well.setExtract(aliquot);
				}
			}
			mips.buildPoolTableRow(plate);
		}
		Plate.paint(wells);
	}
	
	mips.buildPoolTableRow = function(plate)
	{
		var html = '<th></th>';
		var columns = plate.columns;
		var pools = [];	// A 'pool' is a number of columns with the same MIPS_Panel
		var currentPool = {};

		var poolNum = -1;
		for (var colNo = 0; colNo < columns.length; colNo++)
		{
			var col = columns[colNo];
			if (col.MIPS_Panel != currentPool.name)
			{
				// The column has a different MIPS_Panel than the previous one so we create a new pool
				currentPool = {};
				currentPool.name = col.MIPS_Panel;
				currentPool.startCol = colNo;
				currentPool.columnSpan = 1;
				currentPool.hasMultipleMIPSPanels = col.hasMultipleMIPSPanels;
				currentPool.hasMultiplePoolVolumes = col.hasMultiplePoolVolumes;
				pools[pools.length] = currentPool;
				poolNum++;
			}
			else if (currentPool)
			{
				// The column has the same MIPS_Panel as the previous one, we make the pool span across multiple columns
				currentPool.columnSpan++;
				if (col.hasMultipleMIPSPanels) currentPool.hasMultipleMIPSPanels = true;
				if (col.hasMultiplePoolVolumes) currentPool.hasMultiplePoolVolumes = true;
			}
		}
		
		colNo = 0;
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var pool = pools[poolNo];
			var poolClass = 'pool link';
			var poolText = null;
			
			if (pool.columnSpan == 1)
			{
				COLUMN_CLASSES[colNo] = 'pool-left pool-right';
			}
			else
			{
				COLUMN_CLASSES[colNo] = 'pool-left';
				for (var i = 1; i < pool.columnSpan-1; i++)
				{
					COLUMN_CLASSES[colNo+i] = 'pool-middle';
				}
				COLUMN_CLASSES[colNo+pool.columnSpan-1] = 'pool-right';
			}

			if (pool.name)
			{
				if (pool.hasMultiplePoolVolumes) 
				{
					poolClass += ' pool-warning';
					poolText = 'Some aliquots have a different pool volume!';
				}
				poolClass += ' ' + MipsColor.getColor(pool.name);
				html += '<th id="pool.' + poolNo + '" colspan="'+pool.columnSpan+'"';
				html += ' class="'+poolClass+'"';
				html += ' data-start-col="' + pool.startCol + '"';
				html += ' data-end-col="' + (pool.startCol+pool.columnSpan-1) + '"';
				if (poolText) html += ' title="'+poolText+'"';
				html += '>'+Strings.encodeTags(pool.name)+'</th>';
			}
			else
			{
				poolClass = '';
				if (colNo > 0) poolClass += ' nopool-after-pool';
				if (colNo + pool.columnSpan < 12) poolClass += ' nopool-before-pool';
				html += '<th colspan="'+pool.columnSpan+'"';
				html += ' class="'+poolClass+'"';
				html += '></th>';
			}
			colNo += pool.columnSpan;
		}
		
		Doc.element('pool-row').innerHTML = html;
		Doc.show('pool-row');
	}
	
	mips.submit = function()
	{
		var frm = document.forms['reggie'];
		var path = frm.importfile.value;

		var url = '../Mips.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportAliquotsFromFile';
		url += '&path='+encodeURIComponent(path);
		url += '&prefix='+encodeURIComponent(frm.prefix.value);
		url += '&sheet='+encodeURIComponent(frm.sheet.value);
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, mips.submissionResults, 'POST');
	}
	
	mips.submissionResults = function(response)
	{
		window.removeEventListener('beforeunload', mips.confirmLeaveWizard);
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	mips.initElements = function(element, autoInit)
	{
		if (autoInit == 'plate-col')
		{
			Events.addEventHandler(element, 'click', mips.toggleColumn);
			Events.addEventHandler(element, 'mouseover', mips.highlightColumn);
			Events.addEventHandler(element, 'mouseout', mips.highlightColumn);
		}
		else if (autoInit == 'plate-row')
		{
			Events.addEventHandler(element, 'click', mips.toggleRow);
			Events.addEventHandler(element, 'mouseover', mips.highlightRow);
			Events.addEventHandler(element, 'mouseout', mips.highlightRow);
		}
		else if (autoInit == 'plate-well')
		{
			Events.addEventHandler(element, 'click', mips.toggleWell);
		}
	}

	// Toggle the selected status of a single well
	mips.toggleWell = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'col');
		var well = Plate.getWell(row, column);
		Plate.toggleSelected([well]);
	}

	// Toggle the selected status of a complete row
	mips.toggleRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		Plate.toggleSelected(Plate.getRow(row));
	}

	// Toggle the selected status of a complete column
	mips.toggleColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		Plate.toggleSelected(Plate.getColumn(column));
	}
	
	// Toggle the selected status of a pool
	mips.togglePool = function(event)
	{
		var startCol = Data.int(event.target, 'start-col');
		var endCol = Data.int(event.target, 'end-col');
		Plate.toggleSelected(Plate.getPool(startCol, endCol));
	}
	
	// Highlight enable/disable all wells in a column
	mips.highlightColumn = function(event)
	{
		var column = Data.int(event.currentTarget, 'col');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-column', on);
		var wells = Plate.getColumn(column);
		Plate.setHighlight(wells, 'highlight-column', on);
	}

	// Highlight enable/disable all wells in a row
	mips.highlightRow = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var on = event.type == 'mouseover';
		
		Doc.addOrRemoveClass(event.currentTarget, 'highlight-row', on);
		var wells = Plate.getRow(row);
		Plate.setHighlight(wells, 'highlight-row', on);
	}
	
	// Highligt enable/disable all wells in a pool
	mips.highlightPool = function(event)
	{
		var startCol = Data.int(event.target, 'start-col');
		var endCol = Data.int(event.target, 'end-col');
		var on = event.type == 'mouseover';
		Doc.addOrRemoveClass(event.target, 'highlight-pool', on);
		var wells = Plate.getPool(startCol, endCol);
		Plate.setHighlight(wells, 'highlight-pool', on);
	}

	return mips;
}();

Doc.onLoad(ImportPlates.initPage);
Doc.addElementInitializer(ImportPlates.initElements);

var COLUMN_CLASSES = [];
var WellPainter = function()
{
	var painter = {};
	
	// Add class indicators for replicates and QC assigned wells
	painter.getClassNameForWell = function(well)
	{
		var cls = '';
		if (well.extract)
		{
			cls += ' primary';
			if (well.extract.MIPS_Panel) cls += ' ' + MipsColor.getColor(well.extract.MIPS_Panel);
		}
		if (COLUMN_CLASSES[well.column]) cls += ' ' + COLUMN_CLASSES[well.column];
		if (well.column == Plate.columns-1) cls += ' last-child';
		return cls;
	}
	
	painter.getWellText = function(well)
	{
		var text = '';
		if (well.extract)
		{
			// The well contains DNA
			var aliquot = well.extract;
			text += '<div class="name">'+Reggie.softLineBreaks(Strings.encodeTags(aliquot.name || aliquot.prefix))+'</div>';
			
			if (aliquot.MIPS_PoolVolume != null)
			{
				text += '<div class="volume">'+Strings.encodeTags(aliquot.MIPS_PoolVolume)+'µl</div>';
			}
			if (aliquot.MIPS_Panel != null)
			{
				text += '<div class="mips-panel">'+Strings.encodeTags(aliquot.MIPS_Panel)+'</div>';
			}

			if (aliquot.MIPS_Form != null || aliquot.MIPS_NormalTumor != null)
			{
				text += '<div class="mips-form">';
				if (aliquot.MIPS_NormalTumor != null) text += '<span>'+Strings.encodeTags(aliquot.MIPS_NormalTumor)+'</span>';
				if (aliquot.MIPS_Form != null) text += '<span>'+Strings.encodeTags(aliquot.MIPS_Form)+'</span>';
				text += '</div>';
			}
			
			var warningMsg = [];
			if (aliquot.comment)
			{
				text += '<div class="comment">'+Strings.encodeTags(aliquot.comment)+'</div>';
			}
			if (warningMsg.length > 0)
			{
				well.setWarning(warningMsg.join('; '));
			}
		}
		else
		{
			text = 'empty';
		}

		return text;
	}
	
	return painter;
}();
