var FlowCell = function()
{
	var flowcell = {};
	var debug = 0;
	var idField;
	var flowCellsAreValid = false;

	var commentsIsValid = false;
	var flowCellIdIsValid = false;
	var autoAnalyzeIsValid = false;

	// Page initialization
	flowcell.initPage = function()
	{
		// Step 1
		Events.addEventHandler('flowcells', 'change', flowcell.flowCellsOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', flowcell.validateStep1);
		Events.addEventHandler('outcomeSuccess', 'click', flowcell.outcomeOnChange);
		Events.addEventHandler('outcomeFailed', 'click', flowcell.outcomeOnChange);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', flowcell.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', flowcell.validateStep2);
		Events.addEventHandler('startDate', 'change', Wizard.validateDate);
		Events.addEventHandler('externalId', 'change', flowcell.flowCellIdOnChange);
		Events.addEventHandler('comments', 'change', flowcell.commentsOnChange);

		Events.addEventHandler('autoAnalyzeYes', 'click', flowcell.autoAnalyzeOnChange);
		Events.addEventHandler('autoAnalyzeNo', 'click', flowcell.autoAnalyzeOnChange);
		Events.addEventHandler('clusters', 'change', flowcell.clusterOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', flowcell.submit);

		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnprocessedFlowCells&pipeline='+encodeURIComponent(Data.get('page-data', 'pipeline'));
		Wizard.showLoadingAnimation('Loading flow cells...');
		Wizard.asyncJsonRequest(url, flowcell.initializeStep1);
	}


	flowcell.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var flowCells = response.flowCells;
		
		if (flowCells.length > 0)
		{
			for (var fcNo=0; fcNo < flowCells.length; fcNo++)
			{
				var fc = flowCells[fcNo];
				var option = flowcell.getOptionForFlowCell(fc, fcNo);
				frm.flowcells[frm.flowcells.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No flow cells available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
		
		flowcell.flowCellsOnChange();
		flowcell.outcomeOnChange();
		frm.flowcells.focus();
	}
	
	flowcell.getOptionForFlowCell = function(fc, fcNo)
	{
		var name = fc.name +': ';

		var numPools = fc.pools.length;
		var firstPoolNum = flowcell.getPoolNum(fc.pools[0].name);
		var lastPoolNum = flowcell.getPoolNum(fc.pools[fc.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += fc.pools[0].name + ' — ' + fc.pools[fc.pools.length-1].name;
			}
			else
			{
				name += fc.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += fc.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}
		
		name += ' [';
		if (fc.FlowCellType) name += fc.FlowCellType + '; ';
		name += fc.pipeline + ']';

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = fc.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}
		
		var option = new Option(name, fc.id, false, fcNo < 1);
		option.title = title;
		option.flowCell = fc;

		return option;
	}
	
	flowcell.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;	
	}

	flowcell.flowCellsOnChange = function()
	{
		var frm = document.forms['reggie'];	
		flowCellsAreValid = false;
		
		var numSelected = 0;
		for (var i = 0; i < frm.flowcells.length; i++)
		{
			if (frm.flowcells[i].selected) numSelected++;
		}
		
		if (numSelected < 1 || numSelected > 2)
		{
			Wizard.setInputStatus('flowcells', 'invalid', 'Select 1 flow cell');
			return false;
		}
		
		Wizard.setInputStatus('flowcells', 'valid');
		flowCellsAreValid = true;
	}

	flowcell.outcomeOnChange = function()
	{
		var frm = document.forms['reggie'];
		var failed = Doc.element('outcomeFailed').checked;
		Doc.addOrRemoveClass(frm['comments'], 'required', failed);
	}
	
	flowcell.validateStep1 = function(event)
	{
		if (!flowCellsAreValid) event.preventDefault();
	}
	
	flowcell.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		var failed = frm.outcome[1].checked;
		if (failed)
		{
			Wizard.showGoNextConfirmation(true, 'Check to verify registration of failure');
		}

		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');
		
		var fc = frm.flowcells[frm.flowcells.selectedIndex].flowCell;
		frm.comments.value = fc.comments;
		frm.externalId.value = fc.externalId;
		Events.sendChangeEvent('comments');
		Events.sendChangeEvent('externalId');
		Events.sendChangeEvent('startDate');

		frm.externalId.focus();

		// Load clustering protocols
		Reggie.loadProtocols('SEQUENCING_PROTOCOL', 'sequencingProtocol');
		Reggie.loadHardware('SEQUENCER', 'sequencer', 'FLOWCELL_TYPE', fc.FlowCellType);
		
		if (!failed)
		{
			// Load clusters
			var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo&config=demux';
			Doc.addClass('clusters', 'list-loading');
			frm.clusters[0] = new Option('loading...');
			Wizard.asyncJsonRequest(url, flowcell.clustersLoaded);
			Doc.show('auto-analyze-section');
			
			// Check debug by default if debug is set or not on a secure server (=production server)
			frm.debug.checked = debug || location.protocol != 'https:';
		}

	}
	
	flowcell.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		frm.clusters.length = 0;
		var clusters = response.hosts;
		if (clusters.length == 0)
		{
			Doc.element('autoAnalyzeNo').checked = true;
			autoAnalyzeIsValid = true;
			Wizard.setInputStatus('autoAnalyze', 'warning', 'No available clusters');
			return;
		}
		
		Doc.element('autoAnalyzeNo').disabled = false;
		Doc.element('autoAnalyzeYes').disabled = false;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.connection.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
		}
		flowcell.autoAnalyzeOnChange();
	}
	
	flowcell.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		frm.priority.length = 0;
		if (cluster && cluster.priorities && cluster.priorities.length > 0)
		{
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default'], p['default']);
			}
		}
		else
		{
			frm.priority[0] = new Option('default');
		}
	}

	flowcell.autoAnalyzeOnChange = function()
	{
		Wizard.setInputStatus('autoAnalyze');
		autoAnalyzeIsValid = false;
		
		var frm = document.forms['reggie'];
		var yesChecked = Doc.element('autoAnalyzeYes').checked;
		var noChecked = Doc.element('autoAnalyzeNo').checked;
		
		frm.clusters.disabled = !yesChecked;
		frm.priority.disabled = !yesChecked;
		frm.debug.disabled = !yesChecked;

		if (!yesChecked && !noChecked)
		{
			Wizard.setInputStatus('autoAnalyze', 'invalid', 'Must select Yes or No');
			return;
		}
		
		autoAnalyzeIsValid = true;
		Wizard.setInputStatus('autoAnalyze', 'valid');
		
		if (yesChecked) flowcell.clusterOnChange();
	}

	// Comments are required when 'Failure' has been selected
	flowcell.commentsOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		
		commentsIsValid = false;
		Wizard.setInputStatus('comments');
		
		var comments = frm.comments.value;
		if (comments == '')
		{
			var required = frm.comments.className.indexOf('required') != -1;
			if (required)
			{
				Wizard.setInputStatus('comments', 'invalid', 'Missing');
				return;
			}
		}

		Wizard.setInputStatus('comments', 'valid');
		commentsIsValid = true;
	}

	flowcell.flowCellIdOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		
		Wizard.setInputStatus('externalId');
		flowCellIdIsValid = false;
		
		var id = frm.externalId.value;
		if (id == '')
		{
			Wizard.setInputStatus('externalId', 'invalid', 'ID is a required value');
			return;
		}
		
		// Check if ID already exists in DB
		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlowCellByBarcode';
		url += '&barcode='+encodeURIComponent(id);
		Wizard.showLoadingAnimation('Checking flow cell id...');
		Wizard.asyncJsonRequest(url, flowcell.flowCellIdChecked);
	}
	
	flowcell.flowCellIdChecked = function(response)
	{
		var frm = document.forms['reggie'];
		var existing = response.flowCell;
		if (existing != null)
		{
			var id = frm.externalId.value;
			Wizard.setInputStatus('externalId', 'invalid', 'Flow cell with ID='+Strings.encodeTags(id)+' already exists: ' + Strings.encodeTags(existing.name));
			return;
		}
		Wizard.setInputStatus('externalId', 'valid');
		flowCellIdIsValid = true;
	}

	flowcell.validateStep2 = function(event)
	{
		var frm = document.forms['reggie'];
		
		var valid = Wizard.isValid('startDate');
		valid &= flowCellIdIsValid  && commentsIsValid && autoAnalyzeIsValid;
		if (!valid)
		{
			event.preventDefault();
		}
	}


	flowcell.submit = function()
	{
		var frm = document.forms['reggie'];
		var failed = Doc.element('outcomeFailed').checked;
		
		var submitInfo = {};
		submitInfo.failed = failed;
		
		var seqRun = {};
		seqRun.startDate = frm.startDate.value;
		seqRun.operator = frm.sequencingOperator.value;
		seqRun.sequencer = parseInt(frm.sequencer.value, 10);
		seqRun.position = frm.sequencerPosition.value;
		seqRun.protocol = parseInt(frm.sequencingProtocol.value, 10);
		seqRun.comments = frm.comments.value;
		submitInfo.sequencingRun = seqRun;
		
		var fc = frm.flowcells[frm.flowcells.selectedIndex].flowCell;
		fc.FlowCellID = frm.externalId.value;
		submitInfo.flowCell = fc;
		
		if (!failed && Doc.element('autoAnalyzeYes').checked)
		{
			var autoAnalyze = {};
			autoAnalyze.cluster = frm.clusters.value;
			autoAnalyze.priority = parseInt(frm.priority.value);
			autoAnalyze.debug = frm.debug.checked;
			submitInfo.autoAnalyze = autoAnalyze;
		}

		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterSequencingStarted&pipeline='+encodeURIComponent(Data.get('page-data', 'pipeline'));
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, flowcell.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	flowcell.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return flowcell;
}();

Doc.onLoad(FlowCell.initPage);

